#!/usr/bin/perl

use subs;
use strict;
use warnings;

# test.pl
#
# runs a test and check its output

sub run($$$) {
  my ($files, $output, $options1) = @_;
  my $cmd = "$options1 $files.smv >$output 2>&1";

  print LOG "Running $cmd\n";
  system $cmd;
  my $exit_value = $? >> 8;
  my $signal_num = $? & 127;
  my $dumped_core = $? & 128;
  my $failed = 0;

  print LOG "  Exit: $exit_value\n";
  print LOG "  Signal: $signal_num\n";
  print LOG "  Core: $dumped_core\n";

  if($signal_num != 0) {
    $failed = 1;
    print "Killed by signal $signal_num";
    if($dumped_core) {
      print " (code dumped)";
    }
  }

  system "echo EXIT=$exit_value >>$output";
  system "echo SIGNAL=$signal_num >>$output";

  return $failed;
}

sub load($) {
  my ($fname) = @_;

  print $fname;
  open FILE, "<$fname";
  
  my @data = <FILE>;
  close FILE;

  chomp @data;
  return @data;
}

sub test($$$$) {
  my @files;
  my ($name, $test, $options1, $number) = @_;
  my ($model, @results) = load($test);
  my $ext = '.out';
  my $output = $name.$ext.'.'.$number;
  print LOG "Test '$name'\n";
  print LOG "  Input: $model\n";
  print LOG "  Options: $options1\n";
  print LOG "  Output: $output\n";
  print LOG "  Results:\n";
  foreach my $result (@results) {
    print LOG "    $result\n";
  }

  my $failed = run($model, $output, $options1);

  if(!$failed) {
    print LOG "Execution [OK]\n";
    my $included = 1;
    foreach my $result (@results) {
      if($result eq "--") {
	$included = !$included;
      } else {
	my $r;
	system "grep '$result' '$output' >/dev/null";
	$r = ($included ? $? != 0 : $? == 0);
	if($r) {
	  print LOG "$result [FAILED]\n";
	  $failed = 1;
	} else {
	  print LOG "$result [OK]\n";
	}
      }
    }
  } else {
    print LOG "Execution [FAILED]\n";
  }

  print LOG "\n";

  return $failed;
}

sub dirs() {
  my @list;

  opendir CWD, ".";
  @list = grep { !/^\./ && -d "$_" && !/CVS/ } readdir CWD;
  closedir CWD;

  @list = sort @list;

  return @list;
}

if(@ARGV != 0) {
  print "Usage:\n";
  print "  test.pl\n";
  exit 1;
}


#provide the options to ebmc
my $options1 = "ebmc --interpolation ";
print "Loading\n";
my @tests = dirs();
my $count = @tests;
if($count == 1) {
  print "  $count test found\n";
} else {
  print "  $count tests found\n";
}
print "\n";
my $failures1 = 0;
print "Running tests\n";
my $cmdrm = "rm tests.log.minimization";
system $cmdrm;

foreach my $test (@tests) {
  print "Running $test \n";
  chdir $test;

  system "pwd";
  print " options : $options1\n";

  open LOG,">>../tests.log.minimization";
  my $failed1 = test($test, "test.desc", $options1, "minimization");
  close LOG;

  if($failed1) {
      $failures1++;
      print "  [FAILED]\n";
  } else {
      print "  [OK]\n";
  }
print " options : $options1\n";
 chdir "..";

}
print "\n";

if($failures1 == 0) {
  print " Option1 : All tests were successful\n";
} else {
  print "Tests failed\n";
  if($failures1 == 1) {
    print " Option1 :  $failures1 of $count test failed\n";
  } else {
    print " Option1 : $failures1 of $count tests failed\n";
  }
}

exit $failures1;
